package TemplatePreviewSelector::CMS;

use strict;
use MT::Util qw( start_end_month start_end_week start_end_day );

use Data::Dumper;

sub init_request {
    my ($eh, $app) = @_;

    my $core = MT->component("core");
    $core->registry("applications","cms","methods",
        { 'preview_template' => '$TemplatePreviewSelector::TemplatePreviewSelector::CMS::preview_template' });
    1;
}

sub change_html {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
    jQuery('button.preview').click(function() {
        syncEditor();
        jQuery('form#template-listing-form > input[name=__mode]').val('preview_template');
    });
HTML

    $old = quotemeta($old);
    $new = <<HTML;
    jQuery('button.preview').click(function() {
        syncEditor();
        jQuery('form#template-listing-form > input[name=__mode]').val('preview_template2');
    });
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <input type="hidden" name="action_input" value="" />
HTML

    $old = quotemeta($old);
    $new = <<HTML;
  <input type="hidden" name="action_input" value="" />
  <input type="hidden" name="template_id" value="" />
  <input type="hidden" name="author_id" value="" />
  <input type="hidden" name="category_id" value="" />
  <input type="hidden" name="entry_id" value="" />
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
</mt:setvartemplate>

<mt:setvarblock name="html_head" append="1">
HTML

my $plugin = MT->component('TemplatePreviewSelector');

my $preview_template = $plugin->translate('Preview Template');
my $preview_category = $plugin->translate('Preview Category');
my $preview_author = $plugin->translate('Preview Author');
my $preview_entry = $plugin->translate('Preview Entry');
my $preview_page = $plugin->translate('Preview Page');

my $category = $plugin->translate('category');
my $author = $plugin->translate('author');
my $year = $plugin->translate('year');
my $month = $plugin->translate('month');
my $week = $plugin->translate('week');
my $day = $plugin->translate('day');
my $year_message = $plugin->translate('year_message');
my $month_message = $plugin->translate('month_message');
my $week_message = $plugin->translate('week_message');
my $day_message = $plugin->translate('day_message');

    $old = quotemeta($old);
    $new = <<HTML;
<div style="display:none">
<mtapp:listing
   show_actions="0"
   type="templatemap">
<mt:var name="archive_type" setvar="archive_types" append="1">
</mtapp:listing>
</div>

<mt:if name="template_group" eq="archive">

<mt:loop name="archive_types">
<mt:var name="archive_type" setvar="merge" append="1">
</mt:loop>

<div style="margin-top:20px">
$preview_template
<select id="preview_template_id" name="preview_template_id">
<mtapp:listing
   show_actions="0"
   type="templatemap">
<option value="<mt:var name="map_id">"><mt:var name="archive_label"></option>
</mtapp:listing>
</select>
</div>

<mt:if name="archive_types" eq="Individual">

<div style="margin-top:10px">
$preview_entry
<select name="preview_entry_id">
<option value="">Recent Entry</option>
<mtloop name="entries">
   <option value="<mt:var name="entry_id">"><mt:var name="entry_title">(<mt:var name="entry_id">)</option>
</mtloop>
</select>
</div>

<mt:elseif name="archive_types" eq="Page">

<div style="margin-top:10px">
$preview_page
<select name="preview_page_id">
<option value="">Recent Page</option>
<mtloop name="pages">
   <option value="<mt:var name="page_id">"><mt:var name="page_title">(<mt:var name="page_id">)</option>
</mtloop>
</select>
</div>

<mt:elseif name="archive_types" like="Category">

<div id="preview_category_area" style="margin-top:10px">
$preview_category
<select name="preview_cat_id">
<mtloop name="categories">
   <option value="<mt:var name="cat_id">"><mt:var name="cat_label">(<mt:var name="cat_id">)</option>
</mtloop>
</select>
</div>
<div id="preview_date_area" style="margin-top:10px"></div>

<mt:elseif name="archive_types" like="Author">

<div id="preview_author_area" style="margin-top:10px">
$preview_author
<select name="preview_author_id">
<mtloop name="authors">
   <option value="<mt:var name="author_id">"><mt:var name="author_name">(<mt:var name="author_id">)</option>
</mtloop>
</select>
</div>
<div id="preview_date_area" style="margin-top:10px"></div>

<mt:else>

<div id="preview_date_area" style="margin-top:10px"></div>

</mt:if>
</mt:if>

<script>
jQuery('#preview_template_id').change(function(){
    var data = jQuery(this).find(":selected").text();
    change_date_archive_html(data);
    change_category_archive_html(data);
});
jQuery(function(){
    var data = jQuery('#preview_template_id option:selected').text();
    change_date_archive_html(data);
    change_category_archive_html(data);
});
var change_date_archive_html = function(data) {
    if (data.match(/$year/)) {
        jQuery('#preview_date_area').html('$year_message<input id="preview_year" name="preview_year" value="" />')
    } else if (data.match(/$month/)) {
        jQuery('#preview_date_area').html('$month_message<input id="preview_month" name="preview_month" value="" />')
    } else if (data.match(/$week/)) {
        jQuery('#preview_date_area').html('$week_message<input id="preview_week" name="preview_week" value="" />')
    } else if (data.match(/$day/)) {
        jQuery('#preview_date_area').html('$day_message<input id="preview_day" name="preview_day" value="" />')
    } else {
        jQuery('#preview_date_area').empty();
    }
};
var change_category_archive_html = function(data) {
    if (data.match(/$category/)) {
        jQuery('#preview_category_area').show();
    } else {
        jQuery('#preview_category_area').hide();
    }
};
var change_author_archive_html = function(data) {
    if (data.match(/$author/)) {
        jQuery('#preview_author_area').show();
    } else {
        jQuery('#preview_author_area').hide();
    }
};
</script>

</mt:setvartemplate>

<mt:setvarblock name="html_head" append="1">
HTML

    $$template =~ s/$old/$new/;
}

sub add_param {
    my ($cb, $app, $param, $template) = @_;

    my $entry_maps = _populate_entry_loop( $app->blog );
    $param->{entries} = $entry_maps;

    my $page_maps = _populate_page_loop( $app->blog );
    $param->{pages} = $page_maps;

    my $cat_maps = _populate_category_loop( $app->blog );
    $param->{categories} = $cat_maps;

    my $author_maps = _populate_author_loop( $app->blog );
    $param->{authors} = $author_maps;
    1;
}


sub _populate_entry_loop {
    my $blog = shift;

    my @entries = MT::Entry->load( { blog_id => $blog->id },
                                   { limit => 100,
                                     direction => 'descend',
                                     sort => 'authored_on', } );
    my @maps;
    my %types;
    foreach my $entry (@entries) {
        my $map = {};
        $map->{entry_id} = $entry->id;
        $map->{entry_title} = $entry->title;
        push @maps, $map;
    }
#    @maps = sort { $a->cat_id <=> $b->cat_id } @maps;
    return \@maps;
}

sub _populate_page_loop {
    my $blog = shift;

    my @pages = MT::Entry->load( { blog_id => $blog->id, class => 'page' },
                                 { limit => 100,
                                   direction => 'descend',
                                   sort => 'authored_on', } );
    my @maps;
    my %types;
    foreach my $page (@pages) {
        my $map = {};
        $map->{page_id} = $page->id;
        $map->{page_title} = $page->title;
        push @maps, $map;
    }
#    @maps = sort { $a->cat_id <=> $b->cat_id } @maps;
    return \@maps;
}

sub _populate_category_loop {
    my $blog = shift;

    my @categories = MT::Category->load( { blog_id => $blog->id } );
    my @maps;
    my %types;
    foreach my $cat (@categories) {
        my $map = {};
        $map->{cat_id} = $cat->id;
        $map->{cat_label} = $cat->label;
        push @maps, $map;
    }
#    @maps = sort { $a->cat_id <=> $b->cat_id } @maps;
    return \@maps;
}

sub _populate_author_loop {
    my $blog = shift;

    my @authors = MT::Author->load();
    my @maps;
    my %types;
    foreach my $author (@authors) {
        my $map = {};
        my $permission = MT::Permission->load( { author_id => $author->id, blog_id => $blog->id } );
        next if !$permission;
        next if !$permission->can_do('edit_templates');
        $map->{author_id} = $author->id;
        $map->{author_name} = $author->name;
        push @maps, $map;
    }
#    @maps = sort { $a->author_id <=> $b->author_id } @maps;
    return \@maps;
}

sub preview_template {
    my $app     = shift;
    my $q       = $app->param;
    my $blog_id = $q->param('blog_id');
    my $blog    = $app->blog;
    my $id      = $q->param('id');
    my $tmpl;
    my $user_id = $app->user->id;

    return unless $app->validate_magic;

    my $perms = $app->blog ? $app->permissions : $app->user->permissions;
    return $app->return_to_dashboard( redirect => 1 )
        unless $perms || $app->user->is_superuser;
    if ( $perms && !$perms->can_edit_templates ) {
        return $app->return_to_dashboard( permission => 1 );
    }

    # We can only do previews on blog templates. Have to publish
    # the preview file somewhere!
    return $app->errtrans("Invalid request.") unless $blog;

    require MT::Template;
    if ($id) {
        $tmpl = MT::Template->load( { id => $id, blog_id => $blog_id } )
            or return $app->errtrans("Invalid request.");
    }
    else {
        $tmpl = MT::Template->new;
        $tmpl->id(-1);
        $tmpl->blog_id($blog_id);
    }

    my $names = $tmpl->column_names;
    my %values = map { $_ => scalar $app->param($_) } @$names;
    delete $values{'id'} unless $q->param('id');

    ## Strip linefeed characters.
    for my $col (qw( text )) {
        $values{$col} =~ tr/\r//d if $values{$col};
    }
    $tmpl->set_values( \%values );

    my $preview_basename = $app->preview_object_basename;

    my $type         = $tmpl->type;
    my $preview_tmpl = $tmpl;
    my $archive_file;
    my $archive_url;
    my %param;
    my $blog_path       = $blog->site_path;
    my $blog_url        = $blog->site_url;
    my $use_virtual_cat = 0;

    if ( ( $type eq 'custom' ) || ( $type eq 'widget' ) ) {

        # determine 'host' template
        $preview_tmpl = MT::Template->load(
            { blog_id => $blog_id, identifier => 'main_index' } );
        if ( !$preview_tmpl ) {
            return $app->errtrans(
                "Can't locate host template to preview module/widget.");
        }
        my $req = $app->request;

        # stash this module so that it is selected through a
        # MTInclude tag instead of the one in the database:
        my $tmpl_name = $tmpl->name;
        $tmpl_name =~ s/^Widget: // if $type eq 'widget';
        my $stash_id
            = 'template_' . $type . '::' . $blog_id . '::' . $tmpl_name;
        $req->stash( $stash_id, [ $tmpl, $tmpl->tokens ] );
    }
    elsif ( ( $type eq 'individual' ) || ( $type eq 'page' ) ) {
        my $ctx = $preview_tmpl->context;
        my $entry_type = $type eq 'individual' ? 'entry' : 'page';
        my ($obj) = create_preview_content2( $app, $blog, $entry_type, 1, undef, 
            $type eq 'individual' ? $q->param('preview_entry_id') : $q->param('preview_page_id'), 0, 0 );
        $obj->basename($preview_basename);
        $ctx->stash( 'entry', $obj );
        $ctx->{current_archive_type}
            = $type eq 'individual' ? 'Individual' : 'Page';
        if ( ( $type eq 'individual' ) && $blog->archive_path ) {
            $blog_path = $blog->archive_path;
            $blog_url  = $blog->archive_url;
        }
        $archive_file = File::Spec->catfile( $blog_path, $obj->archive_file );
        $archive_url = $obj->archive_url;

        my $archiver
            = MT->publisher->archiver( $ctx->{current_archive_type} );
        my $tparams = $archiver->template_params;
        if ($tparams) {
            $ctx->var( $_, $tparams->{$_} ) for keys %$tparams;
        }
    }
    elsif ( $type eq 'archive' ) {

        # some variety of archive template
        my $ctx = $preview_tmpl->context;
        require MT::TemplateMap;
#        my $map = MT::TemplateMap->load( { template_id => $id } );
        my $map = MT::TemplateMap->load( { id => $q->param('preview_template_id') } );
        if ( !$map ) {
            return $app->error(
                $app->translate("Cannot preview without a template map!") );
        }
        $ctx->{current_archive_type} = $map->archive_type;
        my $archiver = MT->publisher->archiver( $map->archive_type );
        my $tparams  = $archiver->template_params;
        if ($tparams) {
            $ctx->var( $_, $tparams->{$_} ) for keys %$tparams;
        }

        my $cat;
        if ( $archiver->category_based ) {
            $cat = MT->model('category')->load(
                { blog_id => $blog_id, id => $q->param('preview_cat_id')},
                {   sort      => 'id',
                    direction => 'ascend',
                }
            );
            unless ($cat) {
                $use_virtual_cat = 1;
                $cat             = new MT::Category;
                $cat->label( $app->translate("Preview") );
                $cat->basename("preview");
                $cat->parent(0);
                $ctx->stash( 'archive_category', $cat );
            }
            $ctx->stash( 'archive_category', $cat );
        }

        # date
        my $preview_date;
        my $preview_date_end;
        my $date_type;
        if ($q->param('preview_year')) {
            $preview_date = $q->param('preview_year');
            $date_type = 'year';
        }
        if ($q->param('preview_month')) {
            $preview_date = $q->param('preview_month');
            $date_type = 'month';
        }
        if ($q->param('preview_week')) {
            $preview_date = $q->param('preview_week');
            $date_type = 'week';
        }
        if ($q->param('preview_day')) {
            $preview_date = $q->param('preview_day');
            $date_type = 'day';
        }

        if ( $date_type eq 'year' && $preview_date =~ /^\d{4}$/ ) {
            $preview_date .= '0101000000';
            $preview_date_end = '1231235959';
        } elsif ( $date_type eq 'month' && $preview_date =~ /^\d{6}$/ ) {
            ( $preview_date, $preview_date_end ) = start_end_month($preview_date);
        } elsif ( $date_type eq 'week' && $preview_date =~ /^\d{8}$/ ) {
            ( $preview_date, $preview_date_end ) = start_end_week($preview_date);
        } elsif ( $date_type eq 'day' && $preview_date =~ /^\d{8}$/ ) {
            ( $preview_date, $preview_date_end ) = start_end_day($preview_date);
        }

        my @entries
            = create_preview_content2( $app, $blog, $archiver->entry_class, 10,
            $cat, 0, $preview_date, $preview_date_end );
        $ctx->stash( 'entries', \@entries );

        if ( $archiver->date_based ) {
            $ctx->{current_timestamp}     = $preview_date;
            $ctx->{current_timestamp_end} = $preview_date_end;
#            $ctx->{current_timestamp}     = $entries[0]->authored_on;
#            $ctx->{current_timestamp_end} = $entries[$#entries]->authored_on;
        }
        if ( $archiver->author_based ) {
            $ctx->stash( 'author', $app->user );
        }

        my $file
            = MT->publisher->archive_file_for( $entries[0], $blog,
            $map->archive_type, $cat, $map, $ctx->{current_timestamp},
            $app->user );
        $archive_file = File::Spec->catfile( $blog_path, $file );
        $archive_url = MT::Util::caturl( $blog_url, $file );
    }
    elsif ( $type eq 'index' ) {
    }
    else {

        # for now, only index templates can be previewed
        return $app->errtrans("Invalid request.");
    }

    my $orig_file;
    my $path;

    # Default case; works for index templates (other template types should
    # have defined $archive_file by now).
    my $outfile = $preview_tmpl->outfile;
    my ($path_in_outfile) = $outfile =~ m/^(.*\/)/;
    $archive_file = File::Spec->catfile( $blog_path, $outfile )
        unless defined $archive_file;

    ( $orig_file, $path ) = File::Basename::fileparse($archive_file);

    $archive_url = MT::Util::caturl( $blog_url, $orig_file )
        unless defined $archive_url;

    my $file_ext;
    require File::Basename;
    $file_ext = $archive_file;
    if ( $file_ext =~ m/\.[a-z]+$/ ) {
        $file_ext =~ s!.+\.!.!;
    }
    else {
        $file_ext = '';
    }
    $archive_file
        = File::Spec->catfile( $path, $preview_basename . $file_ext );

    my @data;
    $app->run_callbacks( 'cms_pre_preview.template', $app, $preview_tmpl,
        \@data );

    my $has_hires = eval 'require Time::HiRes; 1' ? 1 : 0;
    my $start_time = $has_hires ? Time::HiRes::time() : time;

    my $ctx = $preview_tmpl->context;
    $ctx->var( 'preview_template', 1 );

    my $html = $preview_tmpl->output;
    $param{build_time}
        = $has_hires
        ? sprintf( "%.3f", Time::HiRes::time() - $start_time )
        : "~" . ( time - $start_time );

    unless ( defined($html) ) {
        return $app->error(
            $app->translate(
                "Publish error: [_1]",
                MT::Util::encode_html( $preview_tmpl->errstr )
            )
        );
    }

    # If MT is configured to do 'local' previews, convert all
    # the normal blog URLs into the domain used by MT itself (ie,
    # blog is published to www.example.com, which is a different
    # server from where MT runs, mt.example.com; previews therefore
    # should occur locally, so replace all http://www.example.com/
    # with http://mt.example.com/).
    my ( $old_url, $new_url );
    if ( $app->config('LocalPreviews') ) {
        $old_url = $blog_url;
        $old_url =~ s!^(https?://[^/]+?/)(.*)?!$1!;
        $new_url = $app->base . '/';
        $html =~ s!\Q$old_url\E!$new_url!g;
    }

    my $fmgr = $blog->file_mgr;

    ## Determine if we need to build directory structure,
    ## and build it if we do. DirUmask determines
    ## directory permissions.
    require File::Basename;
    $path =~ s!/$!!
        unless $path eq '/';    ## OS X doesn't like / at the end in mkdir().
    unless ( $fmgr->exists($path) ) {
        $fmgr->mkpath($path);
    }

    if ( $fmgr->exists($path) && $fmgr->can_write($path) ) {
        $param{preview_file} = $preview_basename;
        my $preview_url = $archive_url;
        $preview_url
            =~ s! / \Q$orig_file\E ( /? ) $!/$path_in_outfile$preview_basename$file_ext$1!x;

        # We also have to translate the URL used for the
        # published file to be on the MT app domain.
        if ( defined $new_url ) {
            $preview_url =~ s!^\Q$old_url\E!$new_url!;
        }

        $param{preview_url} = $preview_url;

        $fmgr->put_data( $html, $archive_file );

        # we have to make a record of this preview just in case it
        # isn't cleaned up by re-editing, saving or cancelling on
        # by the user.
        require MT::Session;
        my $sess_obj = MT::Session->get_by_key(
            {   id   => $preview_basename,
                kind => 'TF',                # TF = Temporary File
                name => $archive_file,
            }
        );
        $sess_obj->start(time);
        $sess_obj->save;
    }
    else {
        return $app->error(
            $app->translate(
                "Unable to create preview file in this location: [_1]", $path
            )
        );
    }

    $param{id} = $id if $id;
    $param{new_object} = $param{id} ? 0 : 1;
    $param{name} = $tmpl->name;
    if ( $type ne 'index' ) {
        $q->param( 'build_dynamic', $tmpl->build_dynamic );
        $q->param( 'build_type',    $tmpl->build_type );
    }
    my $cols = $tmpl->column_names;
    for my $col ( ( @$cols, 'save_revision', 'revision-note' ) ) {
        push @data,
            {
            data_name  => $col,
            data_value => scalar $q->param($col)
            };
    }
    $param{template_loop}   = \@data;
    $param{object_type}     = $type;
    $param{use_virtual_cat} = $use_virtual_cat;
    $app->request( 'preview_object', $tmpl );
    return $app->load_tmpl( 'preview_template_strip.tmpl', \%param );
}

sub create_preview_content2 {
    my ( $app, $blog, $type, $number, $cat, $entry_id, $preview_date, $preview_date_end ) = @_;

    my $blog_id     = $blog->id;
    my $entry_class = $app->model($type);
    my $cat_args
        = $cat
        ? { join => MT->model('placement')
            ->join_on( 'id', { category_id => $cat->id } ) }
        : {};
    my $id = $entry_id ? { id => $entry_id } : { status  => MT::Entry::RELEASE() };
    my $range_incl = $preview_date ? { range_incl => { 'authored_on' => 1 } } : {};
    my $authored_on = $preview_date ? { authored_on => [ $preview_date, $preview_date_end ] } : {};
    my @obj = $entry_class->load(
        {   blog_id => $blog_id,
#            status  => MT::Entry::RELEASE(),
            %$authored_on,
            %$id,
        },
        {   limit => $number || 1,
            %$range_incl,
            direction => 'descend',
            'sort'    => 'authored_on',
            %$cat_args,
        }
    );
    return @obj;
}

1;
